//--------------------------------------------------------------------------------------------------
//
// full_duplex_serial_pt_demo.c
//
// This program demonstrates the use of full_duplex_serial_pt
//
// Copyright (c) 2011 Michael Rychlik
// MIT Licensed (see at end of file for exact terms)
//
//--------------------------------------------------------------------------------------------------
#include "stdio.h"
#include "propeller.h"
#include "full_duplex_serial_pt.h"

// This structure is passed to the C cog.
// See full_duplex_serial_pt.h for the definition
volatile struct fds_mailbox mbox;

// Start up a new cog running the full duplex serial cog
// code (which we've placed in the .coguser1 section)
void fdx_start(int tx_pin, int rx_pin, int baud)
{
// <HEATER>Where on earth does this label come from?</HEATER>
    extern unsigned int _load_start_coguser1[];

    // Set up the fdx mail box 
    mbox.txHead = 0;
    mbox.txTail = 0;
    mbox.rxHead = 0;
    mbox.rxTail = 0;
    mbox.rxPin = rx_pin;
    mbox.txPin = tx_pin;
    mbox.baudRate = baud;

    // Start fdx driver cog
    cognew (_load_start_coguser1, &mbox);
}

// Recieve a char from the UART
int fdx_rx()
{
    while (1)
    {
        // Get char from the RX FIFO is it is not empty
        if (mbox.rxHead != mbox.rxTail)
        {
            // Yes: Remove char from RX FIFO 
            return(mbox.rxBuffer[mbox.rxTail++ & BUFFER_MASK]);
        }
     }
}

// Transmit a char through the UART
void fdx_tx(char c)
{
    // Wait if the tx buffer is full
    while((mbox.txHead - mbox.txTail) >= BUFFER_SIZE - 1);
    // Put char in buffer.
    mbox.txBuffer[mbox.txHead++ & BUFFER_MASK] = c;	
}

// Put s a string to the UART
fdx_puts(char* s)
{
    while(*s != 0)
    {
        fdx_tx(*s++);        
    }
}

char jackSays[] = "All work and no play make Jack a dull bot.\n";

// Main code
// This is the code running in the LMM cog (cog 0).
// It launches another cog to actually run the full duplex serial code
void main (int argc,  char* argv[])
{
    printf(jackSays);

    // Give up the pins for the fds driver
    _DIRA = 0;
    _OUTA = 0;

    // Start the new cog
    fdx_start(30, 31, 38400);

    // Print message and echo recieved chars
    while(1)
    {
        fdx_puts(jackSays);    
        fdx_tx(fdx_rx());
        fdx_tx(' ');
    }
}
//--------------------------------------------------------------------------------------------------
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files
// (the "Software"), to deal in the Software without restriction,
// including without limitation the rights to use, copy, modify, merge,
// publish, distribute, sublicense, and/or sell copies of the Software,
// and to permit persons to whom the Software is furnished to do so,
// subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//--------------------------------------------------------------------------------------------------
